#include "applied_bonus_points.hpp"

template <GradingType Grading>
void AppliedBonusPoints<Grading>::recalculate() {

    const auto &bonusRef = std::get<const BonusPoints &>(this->mParentRefs);
    const auto &maxClosedRef =
        std::get<const ChangedMaxPoints<GradingType::Closed> &>(
            this->mParentRefs);
    const auto &maxOpenRef =
        std::get<const ChangedMaxPoints<GradingType::Open> &>(
            this->mParentRefs);

    mBonusPointsPerStudent.clear();

    // Calculate how to distribute the bonus points to the open/closed part
    const auto totalPoints = maxClosedRef.mChangedMaxPointBoundary +
                             maxOpenRef.mChangedMaxPointBoundary;

    const auto fraction =
        std::get<const ChangedMaxPoints<Grading> &>(this->mParentRefs)
            .mChangedMaxPointBoundary /
        totalPoints;

    assert(fraction >= 0. && fraction <= 1.);

    for (const auto &s : bonusRef.mTotalBonusPointsPerStudent) {

        mBonusPointsPerStudent.emplace_back(s.first, s.second * fraction);
    }
}

template <GradingType Grading>
Points AppliedBonusPoints<Grading>::getNrBonusPoints(
    const QString &regNr) const noexcept {
    return impl::getNrBonusPoints(mBonusPointsPerStudent, regNr);
}

template struct AppliedBonusPoints<GradingType::Closed>;
template struct AppliedBonusPoints<GradingType::Open>;
