#pragma once

#include "node.hpp"

#include "data/csv_dump.hpp"

#include <QList>
#include <QString>

struct StudentData : Node<> {
    using Node<>::Node;

    /**
     * \brief Interprets the CsvDump as an RWTHOnline-csv-file and copies all
     * necessary info. Also runs sanity checks on the given data and throws on
     * error.
     *
     * \exception std::invalid_argument A sanity check on the given csv file
     * failed, see the description in the exception for details, or a std::stoi
     * conversion failed.
     * \exception std::out_of_range Thrown by std::stoi.
     */
    explicit StudentData(const CsvDump &c) noexcept(false);

    QList<QString> mRegistrationNr;
    QList<int> mAttemptNr;

    // Store the entire csv dump so that we can more easily export it later
    CsvDump mRwthOnline;

private:
    // internal helper functions
    void reserveArrays(qsizetype nrStudents) noexcept;
    void insertData(const CsvDump &c, qsizetype regIdx,
                    qsizetype attemptIdx) noexcept(false);
};
