#pragma once

#include <array>

namespace str {
namespace ui {
namespace boundary {
constexpr auto autoCheckbox = "auto";
constexpr auto closedInfoLabel =
    "Due to regulations, some grade boundaries cannot be changed. See the "
    "grading guidelines available from the help menu of this app for more "
    "information.";
constexpr auto min = "<b>Min</b>";
constexpr auto max = "<b>Max</b>";
constexpr auto grade = "<b>%1</b>";
constexpr auto maxSmaller = "-";

constexpr auto requirePassAllCheckbox =
    "Only pass the exam if all\nexam parts are passed individually";
constexpr auto round1 = "Round grade boundaries: 1";
constexpr auto round0_5 = "Round grade boundaries: 0.5";
constexpr auto round0_25 = "Round grade boundaries: 0.25";
constexpr auto roundDown = "Round grade boundaries down";
constexpr auto roundUp = "Round grade boundaries up";
constexpr auto roundNearest = "Round grade boundaries nearest";
constexpr auto openTab = "Open";
constexpr auto closedTab = "Closed";
} // namespace boundary

namespace input {
constexpr auto guidelineHint =
    "Document that details how to grade multiple choice exams: <a "
    "href=\"https://www.rwth-aachen.de/global/"
    "show_document.asp?id=aaaaaaaaacxybai\">German</a> / <a "
    "href=\"https://www.rwth-aachen.de/global/"
    "show_document.asp?id=aaaaaaaaaejylxi\">English</a>";
constexpr auto csvTab = "Standard (.csv)";
constexpr auto d2rTab = "All in one (.d2r)";
inline constexpr std::array helpTexts = {
    "CSV export of results", "CSV export of \"List of Candidates\"",
    "CSV export of only \"Bonus points\" from moodle",
    "Import previously exported d2r-file from this app"};
constexpr auto csvFilter =
    "Comma-separated values (CSV) file (*.csv);;All files (*.*)";
constexpr auto d2rFilter = "All-in-one (D2R) file (*.d2r);;All files (*.*)";
constexpr auto importButton = "Import...";
constexpr auto radioRoundQuestion = "Grade boundary step:";
constexpr auto radioRound1 = "1.0 point";
constexpr auto radioRound0_5 = "0.5 points";
constexpr auto radioRound0_25 = "0.25 points";
} // namespace input
} // namespace ui
namespace title {
// Window titles
constexpr auto readingFileWarning = "Reading file warnings";
constexpr auto readingFileError = "Reading file failed";
constexpr auto boundaryWindow = "Options";
constexpr auto inputWindow = "Select files...";
} // namespace title
namespace error {
// Error prefix for any error when parsing a dynexite file
constexpr auto defaultDynexite =
    "Please download the CSV results file from the \"Download the "
    "results\" section from dynexite.\n\n";

// Error prefix for any error when parsing an RWTH Online file
constexpr auto defaultRwthOnline =
    "Please download the export from RWTH Online as a CSV file with "
    "encoding-setting ISO 8859-1.\n\n";

// Error prefix for any error when parsing a bonus point file
constexpr auto defaultBonus =
    "Please use a properly formatted bonus point file with at "
    "least the columns \"Matrikelnummer\" and \"Bonuspunkte "
    "(Punkte)\" from moodle.\n\n";

// Error prefix for any error when parsing a d2r file
constexpr auto defaultD2R =
    "Reading the d2r file has generated errors. Please select a file that you "
    "previously exported with this app.\n\n";
constexpr auto d2rDynexite = "While parsing the embedded dynexite file:\n\n";
constexpr auto d2rRwthOnline =
    "While parsing the embedded RWTHonline file:\n\n";
constexpr auto d2rBonus = "While parsing the embedded bonus point file:\n\n";

// Error when parsing an empty or non-text file
constexpr auto empty = "File is empty or of the wrong file type.";

// Error when there are no header columns
constexpr auto noHeader = "No header.";

// Error when there are no rows beside the header
constexpr auto noData = "No data rows.";

// Error when there are no rows beside the header in a dynexite file with
// content besides the identifier
constexpr auto noStudents = "No student attended?";

constexpr auto unexpectedDynexite = "You selected a dynexite file.";
constexpr auto unexpectedRwthOnline = "You selected an RWTH Online file.";
constexpr auto unexpectedBonus = "You selected a bonus point file.";

constexpr auto invalidColumnNr = "Unexpected number of columns: ";

constexpr auto insertFailed = "Inserting data failed.";

constexpr auto noIdColumn = "No registration number column found.";
constexpr auto noBonusPointColumn = "No bonus point column found.";
constexpr auto noAttemptNrColumn = "No attempt number column found.";

constexpr auto bonusArraySizeMismatch =
    "Internal error while reading bonus points (array size mismatch).";
constexpr auto rwthArraySizeMismatch =
    "Internal error while reading student info (array size mismatch).";

constexpr auto readingFileWarning =
    "Reading the file generated the following warnings:\n";

// Warnings when there is little overlap between students in the different input
// files
constexpr auto lowStudentIntersection =
    "Are you sure you selected files from the same exam?";
constexpr auto lowStudentIntersectionFewRwth =
    "\n\nOnly %1% (%2 of %3) of students in the "
    "dynexite file can be found in the "
    "RWTHonline file.";
constexpr auto lowStudentIntersectionNoneRwth =
    "\n\nNone of the students in the dynexite file can be found in "
    "the RWTHonline file.";
constexpr auto lowStudentIntersectionFewBonus =
    "\n\nOnly %1% (%2 of %3) of students in the "
    "dynexite file can be found in the "
    "bonus point file.";
constexpr auto lowStudentIntersectionNoneBonus =
    "\n\nNone of the students in the dynexite file can be found in "
    "the bonus point file.";

constexpr auto addBonusPointsFailed =
    "Failed to add %1 bonus points to student with "
    "registration number %2";

constexpr auto d2rHeader = "Invalid d2r header.";

constexpr auto csvDelimiter =
    "Error while parsing the CSV file: No valid delimiter found";

} // namespace error

namespace info {
constexpr auto sixtyPercentHover1 =
    "Normally, the passing mark (Bestehensgrenze) of an exam should "
    "not be set above the required knowledge of 50% of the obtainable points. "
    "However, since it can be assumed that a candidate who has the required "
    "knowledge (within the closed part) can guess the remaining questions and "
    "hence gain additional points. Therefore the passing mark in the closed "
    "part should not exceed 60% of the achievable points.";

constexpr auto sixtyPercentHover2 =
    "Here, as a result of the legal situation (see Guidelines on "
    "Multiple-Choice Exams), the passing mark should not exceed ";

constexpr auto studentsTableTabLegend =
    R"(<font color="black"><b>Legend:</b></font><br>
<font color="green">Light green: Passed with 1.0</font><br>
<font color="darkGreen">Dark green: Passed</font><br>
<font color="red">Red: Not passed</font>
)";

} // namespace info

namespace control {
constexpr auto richTextStart = "<html><head/><body><p>";
constexpr auto richTextEnd = "</p></body></html>";
} // namespace control

namespace fileExport {
constexpr auto rwthOnlineRemark = "%1 questions: %2 pts (grade: %3)";
}
} // namespace str
