#include "help_widget.hpp"

#include <QMessageBox>
#include <QStyle>

#include <utility>

HelpWidget::HelpWidget(QWidget *parent, QString title, QString text)
    : QPushButton(parent), mTitle(std::move(title)), mText(std::move(text)) {
    setFlat(true);

    setIcon(style()->standardIcon(QStyle::SP_MessageBoxQuestion));

    setToolTip(mText);
    // Don't automatically hide the tooltip after a few seconds
    setToolTipDuration(0);

    connect(this, &HelpWidget::clicked, this, &HelpWidget::questionMarkClicked);
}

void HelpWidget::setText(const QString &text) {
    mText = text;
    setToolTip(mText);
}

QString HelpWidget::text() const { return mText; }

void HelpWidget::setTitle(const QString &title) { mTitle = title; }

QString HelpWidget::title() const { return mTitle; }

void HelpWidget::questionMarkClicked() const {
    QMessageBox::information(parentWidget(), mTitle, mText);
}
