import sys


regIdMap = {}
regNrCounter = 1


def replace_key(line, start, end, new_value):
    oldValue = line[start:end].strip('\"')
    line = line[:start] + str(new_value) + line[end:]
    return line, oldValue


def nth_delim(line, delim, nr):
    lastLocation = line.find(delim)
    for i in range(0, nr):
        lastLocation = line.find(delim, lastLocation + 1)
    return lastLocation


# TODO: replace exercise names?
# TODO: randomize line order
# TODO: instructions in README
def process_dynexite(path, delim):
    global regIdMap
    global regNrCounter
    new_content = ""

    with open(path, 'r') as file:
        # skip header line
        new_content += file.readline()
        iterfile = iter(file)

        for num, line in enumerate(iterfile, 1):
            # skip empty lines
            if line.count(delim) == 0:
                continue

            # delimiter sanity check
            if line.count(delim) < 13:
                print(f"Warning: Too few ('{line.count(delim)}') delimiters in line '{str(num)}' in the dynexite file!")
                continue

            # replace AttemptID
            line = replace_key(line, 0, line.find(delim), "attemptid")[0]

            # replace registration number, store mapping in regIdMap
            start = line.find(delim) + 1
            end = line.find(delim, start)
            line, oldRegNr = replace_key(line, start, end, regNrCounter)
            regIdMap[oldRegNr] = regNrCounter
            regNrCounter += 1

            new_content += line

    with open(path[:-4] + "_pseudo.csv", 'w') as file:
        file.write(new_content)


def process_rwth(path, delim):
    global regIdMap
    global regNrCounter
    new_content = ""

    with open(path, 'r') as file:
        # skip header line (TODO: validate order of headers)
        new_content += file.readline()
        iterfile = iter(file)

        for num, line in enumerate(iterfile, 1):
            # skip empty lines
            if line.count(delim) == 0:
                continue

            # delimiter sanity check
            if line.count(delim) != 23:
                print(f"Warning: Wrong number ('{line.count(delim)}') of delimiters in line '{str(num)}' in the rwthonline file!")
                continue

            # replace STUDY_PROGRAMME
            line = replace_key(line, 0, nth_delim(line, delim, 0), "Bachelorstudium - Informatik")[0]

            # replace CODE_OF_STUDY_PROGRAMME
            line = replace_key(line, nth_delim(line, delim, 0) + 1, nth_delim(line, delim, 1), "1234 56 789")[0]

            # replace Studienplan_Version
            line = replace_key(line, nth_delim(line, delim, 1) + 1, nth_delim(line, delim, 2), "2018")[0]

            # replace SPO_KONTEXT
            line = replace_key(line, nth_delim(line, delim, 2) + 1, nth_delim(line, delim, 3), "[2018] Informatik > Fach")[0]

            # replace REGISTRATION_NUMBER
            oldRegNr = line[nth_delim(line, delim, 3) + 1:nth_delim(line, delim, 4)].strip('\"')
            if oldRegNr in regIdMap:
                newRegNr = regIdMap[oldRegNr]
            else:
                newRegNr = regNrCounter
                regIdMap[oldRegNr] = regNrCounter
                regNrCounter += 1
            line = line[:nth_delim(line, delim, 3) + 1] + str(newRegNr) + line[nth_delim(line, delim, 4):]

            # replace FAMILY_NAME_OF_STUDENT
            line = replace_key(line, nth_delim(line, delim, 4) + 1, nth_delim(line, delim, 5), "Last name")[0]

            # replace FIRST_NAME_OF_STUDENT
            line = replace_key(line, nth_delim(line, delim, 5) + 1, nth_delim(line, delim, 6), "First name")[0]

            # replace GESCHLECHT
            line = replace_key(line, nth_delim(line, delim, 6) + 1, nth_delim(line, delim, 7), "X")[0]

            # replace DATE_OF_ASSESSMENT
            line = replace_key(line, nth_delim(line, delim, 7) + 1, nth_delim(line, delim, 8), "01.01.1971")[0]

            # Don't replace GUEL_U_AKTUELLE_ANTRITTE_SPO!

            # replace GRADE
            line = replace_key(line, nth_delim(line, delim, 9) + 1, nth_delim(line, delim, 10), "")[0]

            # replace REMARK
            line = replace_key(line, nth_delim(line, delim, 10) + 1, nth_delim(line, delim, 11), "")[0]

            # replace Number_Of_The_Course
            line = replace_key(line, nth_delim(line, delim, 11) + 1, nth_delim(line, delim, 12), "12AB34567")[0]

            # replace SEMESTER_OF_The_COURSE
            line = replace_key(line, nth_delim(line, delim, 12) + 1, nth_delim(line, delim, 13), "71W")[0]

            # replace COURSE_TITLE
            line = replace_key(line, nth_delim(line, delim, 13) + 1, nth_delim(line, delim, 14), "Kurs")[0]

            # replace Examiner
            line = replace_key(line, nth_delim(line, delim, 14) + 1, nth_delim(line, delim, 15), "Mustermann Max")[0]

            # replace Start_Time
            line = replace_key(line, nth_delim(line, delim, 15) + 1, nth_delim(line, delim, 16), "09:00")[0]

            # replace TERMIN_ORT
            line = replace_key(line, nth_delim(line, delim, 16) + 1, nth_delim(line, delim, 17), "ZuseLab C1 (2315|107)")[0]

            # replace DB_Primary_Key_Of_Exam
            line = replace_key(line, nth_delim(line, delim, 17) + 1, nth_delim(line, delim, 18), "1234567")[0]

            # replace DB_Primary_Key_Of_Candidate
            line = replace_key(line, nth_delim(line, delim, 18) + 1, nth_delim(line, delim, 19), "9876543")[0]

            # replace COURSE_GROUP_NAME
            line = replace_key(line, nth_delim(line, delim, 19) + 1, nth_delim(line, delim, 20), "")[0]

            # replace FILE_REMARK
            line = replace_key(line, nth_delim(line, delim, 20) + 1, nth_delim(line, delim, 21), "")[0]

            # replace EMAIL_ADDRESS
            line = replace_key(line, nth_delim(line, delim, 21) + 1, nth_delim(line, delim, 22), "some-name@rwth-aachen.de")[0]

            # replace ECTS_GRADE
            line = replace_key(line, nth_delim(line, delim, 22) + 1, line.find('\n'), "")[0]

            new_content += line

    with open(path[:-4] + "_pseudo.csv", 'w') as file:
        file.write(new_content)


def process_bonus(path, delim):
    global regIdMap
    global regNrCounter
    new_content = ""

    with open(path, 'r') as file:
        # skip header line (TODO: validate order of headers)
        new_content += file.readline()
        iterfile = iter(file)

        for num, line in enumerate(iterfile, 1):
            # skip empty lines
            if line.count(delim) == 0:
                continue

            # delimiter sanity check
            if line.count(delim) != 5:
                print(f"Warning: Wrong number ('{line.count(delim)}') of delimiters in line '{str(num)}' in the bonus file!")
                continue

            # replace Vorname
            line = replace_key(line, 0, nth_delim(line, delim, 0), "Max")[0]

            # replace Nachname
            line = replace_key(line, nth_delim(line, delim, 0) + 1, nth_delim(line, delim, 1), "Mustermann")[0]

            # replace Matrikelnummer
            oldRegNr = line[nth_delim(line, delim, 1) + 1:nth_delim(line, delim, 2)].strip('\"')
            if oldRegNr in regIdMap:
                newRegNr = regIdMap[oldRegNr]
            else:
                newRegNr = regNrCounter
                regIdMap[oldRegNr] = regNrCounter
                regNrCounter += 1
            line = line[:nth_delim(line, delim, 1) + 1] + str(newRegNr) + line[nth_delim(line, delim, 2):]

            # replace LMS-ID
            line = replace_key(line, nth_delim(line, delim, 2) + 1, nth_delim(line, delim, 3), "abc_123defgh")[0]

            # Don't replace Bonuspunkte (Punkte)!

            # replace Zuletzt aus diesem Kurs geladen
            line = replace_key(line, nth_delim(line, delim, 4) + 1, line.find('\n'), "1234567890")[0]

            new_content += line

    with open(path[:-4] + "_pseudo.csv", 'w') as file:
        file.write(new_content)


if __name__ == "__main__":
    if len(sys.argv) != 5:
        print("Usage: python pseudonymize.py <delimiter dynexite> <delimiter rwth> <delimiter bonus> <filename>")
        sys.exit(1)

    dynexite_delimiter = sys.argv[1]
    rwth_delimiter = sys.argv[2]
    bonus_delimiter = sys.argv[3]
    dynexite_path = "dynexite/" + sys.argv[4] + ".csv"
    rwth_path = "rwthonline/" + sys.argv[4] + ".csv"
    bonus_path = "bonus/" + sys.argv[4] + ".csv"

    process_dynexite(dynexite_path, dynexite_delimiter)
    process_rwth(rwth_path, rwth_delimiter)
    process_bonus(bonus_path, bonus_delimiter)
    print(f"File '{sys.argv[4]}' has been processed successfully!")
